# babel-plugin-transform-function-partial-application

[![Travis build status](http://img.shields.io/travis/gajus/babel-plugin-transform-function-partial-application/master.svg?style=flat-square)](https://travis-ci.org/gajus/babel-plugin-transform-function-partial-application)
[![NPM version](http://img.shields.io/npm/v/babel-plugin-transform-function-partial-application.svg?style=flat-square)](https://www.npmjs.org/package/babel-plugin-transform-function-partial-application)
[![Canonical Code Style](https://img.shields.io/badge/code%20style-canonical-blue.svg?style=flat-square)](https://github.com/gajus/canonical)

Uses [function-bind syntax](https://github.com/babel/babel/tree/master/packages/babel-plugin-syntax-function-bind) to transpile bind expressions to partially applied call expressions. The first parameter of the function is set to the bind context.

Input:

```js
apple
  ::foo('a')
  ::bar('b');
```

Output:

```
bar(foo(apple, 'a'), 'b');
```

> ***BIG WARNING***: This is a proof-of-concept. See [Motivation](#motivation).

## Examples

### Using existing functional programming utilities

```js
import {
  assocPath
} from 'ramda';

({
  name: 'babel-plugin-transform-function-partial-application'
})
  ::assocPath(['repository', 'type'], 'git')
  ::assocPath(['repository', 'url'], 'https://github.com/gajus/babel-plugin-transform-function-partial-application');

// {
//   name: 'babel-plugin-transform-function-partial-application',
//   repository: {
//     type: 'git',
//     url: 'https://github.com/gajus/babel-plugin-transform-function-partial-application'
//   }
// }
```

### Using utility functions to construct Promises

```js
const map = (promise, callback) => {
  return promise
    .then((values) => {
      const mappedValues = [];

      for (const key in values) {
        if (values.hasOwnProperty(key)) {
          mappedValues.push(callback(values[key], key, values));
        }
      }

      return mappedValues;
    });
};

Promise
  .resolve([
    'foo',
    'bar',
    'baz'
  ])
  ::map((currentValue, index) => {
    return index + ':' + currentValue;
  })
  .then((values) => {
    values;

    // [
    //   '0:foo',
    //   '1:bar',
    //   '2:baz'
    // ]
  });
```

## Motivation

Enable use of the existing functional programming libraries in a way thats easy to debug and performant.
